/***************************************************************************//**
 *   @file   Communication.c
 *   @brief  Implementation of Communication Driver for RENESAS RX62N
 *           Processor.
 *   @author DBogdan (dragos.bogdan@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 570
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "Communication.h"

/***************************************************************************//**
 * @brief Initializes the SPI communication peripheral.
 *
 * @param lsbFirst - Transfer format (0 or 1).
 *                   Example: 0x0 - MSB first.
 *                            0x1 - LSB first.
 * @param clockFreq - SPI clock frequency (Hz).
 *                    Example: 1000 - SPI clock frequency is 1 kHz.
 * @param clockPol - SPI clock polarity (0 or 1).
 *                   Example: 0x0 - idle state for SPI clock is low.
 *	                          0x1 - idle state for SPI clock is high.
 * @param clockPha - SPI clock phase (0 or 1).
 *                   Example: 0x0 - data is latched on the leading edge of SPI
 *                                  clock and data changes on trailing edge.
 *                            0x1 - data is latched on the trailing edge of SPI
 *                                  clock and data changes on the leading edge.
 *
 * @return 0 - Initialization failed, 1 - Initialization succeeded.
*******************************************************************************/
unsigned char SPI_Init(unsigned char lsbFirst,
                       unsigned long clockFreq,
                       unsigned char clockPol,
                       unsigned char clockPha)
{
	unsigned long spbrValue = 0;

	MISOB_B_PIN_IN;
	MOSIB_B_PIN_OUT;
	ADAS1000_CS_PIN_OUT;
	ADAS1000_CS_PIN_HIGH;
	
	if(clockFreq <= 6150000)
	{
		spbrValue = 2;	// 8 MHz.
	}
	else
	{
		spbrValue = 0;	// 24 MHz.
	}
	R_SPI_Create(1,							// Channel selection.
				 PDL_SPI_MODE_SPI_MASTER |	// Connection mode.
				 PDL_SPI_PIN_B |			// A pins for signal MISO, MOSI, RSPCK, SSL0, SSL1, SSL2 and SSL3.
				 PDL_SPI_PIN_MOSI_IDLE_HIGH,// The MOSI output state when no SSLn pin is active.
				 PDL_SPI_FRAME_1_1,			// Frame configuration selection.
				 PDL_NO_DATA,				// Extended timing control - default settings.
                 0x80000000 | spbrValue);	// Bit rate or register value.
	R_SPI_Command(1,						// Channel selection.
				  0,						// Command selection.
				  PDL_SPI_CLOCK_MODE_3 |	// Clock is high when idle; data is sampled on the rising edge.
				  PDL_SPI_DIV_1 |			// Bit rate (specified for R_SPI_Create) : 1.
				  PDL_SPI_LENGTH_8 |	    // The number of bits in the frame transfer.
				  PDL_SPI_MSB_FIRST,	    // Most-significant bit first.
                  PDL_NO_DATA);			    // Extended timing control - default settings.

    return 1;
}

/***************************************************************************//**
 * @brief Writes data to SPI.
 *
 * @param data - data represents the write buffer.
 * @param bytesNumber - Number of bytes to write.
 *
 * @return Number of written bytes.
*******************************************************************************/
unsigned char SPI_Write(unsigned char* data,
						unsigned char bytesNumber)
{
	unsigned char byte = 0;
	
	/* Set the Chip select pin to low to begin transmission. */
	PORTE.DR.BIT.B4 = 0;
	/* Enable the generation of RSPI transmit interrupt requests, Master mode. */
	RSPI1.SPCR.BYTE = 0x48;
	for(byte = 0; byte < bytesNumber; byte++)
	{
		/* Load data into SPDR register. */
		RSPI1.SPDR.LONG = data[byte];
		/* Wait for the RSPI to become idle. */
		while(RSPI1.SPSR.BIT.IDLNF);
		/* Read the SPDR register to clear interrupt flag. */
		(void)RSPI1.SPDR.LONG;
	}
	/* Set the Chip select pin to high to terminate transmission. */
	PORTE.DR.BIT.B4 = 1;

	return bytesNumber;
}

/***************************************************************************//**
 * @brief Reads data from SPI.
 *
 * @param data - Data represents the read buffer.
 * @param bytesNumber - Number of bytes to read.
 *
 * @return Number of read bytes.
*******************************************************************************/
unsigned char SPI_Read(unsigned char* data,
					   unsigned char bytesNumber)
{
	unsigned char byte = 0;
	
	/* Set the Chip select pin to low to begin transmission. */
	PORTE.DR.BIT.B4 = 0;
	/* Enable the generation of RSPI transmit interrupt requests, Master mode. */
	RSPI1.SPCR.BYTE = 0x48;
	for(byte = 0; byte < bytesNumber; byte++)
	{
		/* Load data into SPDR register. */
		RSPI1.SPDR.LONG = 0;
		/* Wait for the RSPI to become idle. */
		while(RSPI1.SPSR.BIT.IDLNF);
		/* Read the SPDR register to clear interrupt flag. */
		data[byte] = (unsigned char)RSPI1.SPDR.LONG;
	}
	/* Set the Chip select pin to high to terminate transmission. */
	PORTE.DR.BIT.B4 = 1;

	return bytesNumber;
}

/***************************************************************************//**
 * @brief Initializes the SCI communication peripheral.
 *
 * @return status - Result of the initialization procedure.
 *                  Example: 1 - Iinitialization was successful;
 *                           0 - Initialization was unsuccessful.
*******************************************************************************/
unsigned char SCI_Init(void)
{
	unsigned char status = 1;
	
	R_SCI_Set(PDL_SCI_PIN_SCI2_B);			// PDL_SCI_PIN_SCI2_B pins.
	R_SCI_Create(2,							// Channel 2.
				 PDL_SCI_ASYNC |			// Asynchronous operation mode.
				 PDL_SCI_TX_CONNECTED |		// The TX output is required.
				 PDL_SCI_RX_DISCONNECTED |	// The RX input is not required.
				 PDL_SCI_CLK_INT_IO |		// The SCK pin functions as an I/O pin.
				 PDL_SCI_8_BIT_LENGTH |		// 8-bit data length.
				 PDL_SCI_PARITY_NONE  |		// No parity bit.
				 PDL_SCI_STOP_1,			// One stop bit.
				 115200,					// The transfer bit rate.
				 1);						// Interrupt priority. 
	
	return status;
}

/***************************************************************************//**
 * @brief Writes data to SCI.
 *
 * @return Number of written bytes.
*******************************************************************************/
unsigned char SCI_Write(unsigned char* data,
						unsigned char bytesNumber)
{
	R_SCI_Send(2,			// Channel selection.
			   PDL_NO_DATA,	// Channel configuration (and Target Station ID).
			   data,		// Data start address.
			   bytesNumber,	// Data count.
			   PDL_NO_FUNC);// Callback function.

	return bytesNumber;
}
